from os.path import exists
from sys import exit

import click

from .bitbucket import command as bitbucket
from .github import command as github
from ...conf import REPOSITORY_PATH


@click.group("init", short_help="Initializes the repo.")
def cli():
    if exists(REPOSITORY_PATH):
        click.echo(f"Project already exists in '{REPOSITORY_PATH}'")
        exit()
    else:
        click.echo(f"Creating seki project in '{REPOSITORY_PATH}'...")


cli.add_command(bitbucket)
cli.add_command(github)



import click

from ...utils.drone import run
from ...utils.git_cli import get_repo, get_repo_path_from_origin


def is_enabled(repository):
    click.echo("Check if drone is enabled...")

    result = run([
        "repo",
        "info",
        repository,
        "--format",
        "{{.Active}}"
    ])

    return "true" in result


def enable_repository(repository):
    click.echo("Enabling drone...")
    # Drone sync
    result = run([
        "repo",
        "sync"
    ])

    if repository not in result:
        click.echo("Drone sync failed")
        exit()

    # Enable project
    result = run([
        "repo",
        "enable",
        repository
    ])

    if "Successfully activated repository" not in result:
        click.echo("Not able to activate repository in drone")
        exit()


@click.command("drone", short_help="Activates drone.")
def cli():
    repo = get_repo()

    repo_path = get_repo_path_from_origin(repo)

    if not is_enabled(repo_path):
        enable_repository(repo_path)
    else:
        click.echo("Drone already enabled.")



import click

from .run import generate_drone_yml, dump_drone_yml
from ...commands.shared import prepare_project, commit_changes
from ...conf import DRONE_PATH


@click.command("run", short_help="Quick run tool in pipeline.")
@click.argument("image")
@click.option("--args", help="Arguments for docker image.")
@click.option("--telegram", is_flag=True, help="Notify on telegram build result.")
@click.option("--cron", type=click.Choice(["@hourly", "@daily", "@weekly", "@monthly", "@yearly"]))
def cli(image, args, telegram, cron):
    prepare_project(cron)

    drone_yml = generate_drone_yml(image, args, telegram)

    dump_drone_yml(DRONE_PATH, drone_yml)

    commit_changes()



import sys
from datetime import datetime
from hashlib import md5

import click

from ..conf import DRONE_PATH
from ..utils.drone import run as run_drone
from ..utils.git_cli import get_repo, commit, push, create_branch, checkout, get_repo_path_from_origin


def generate_header():
    current_time = datetime.now().isoformat(" ", "seconds")

    args = " ".join(sys.argv)

    return args + " - " + current_time


def add_drone_cron(repository, branch, cron_expr):
    click.echo("Creating cron job in drone...")

    result = run_drone([
        "cron",
        "add",
        "--branch",
        branch,
        repository,
        branch,  # name for cron
        cron_expr
    ])

    if result != "":
        click.echo("Not able to activate repository in drone")


def prepare_project(cron):
    repo = get_repo()

    checkout(repo, "master")

    if cron:
        branch_name = generate_header()

        branch_name_hash = md5(branch_name.encode("utf-8")).hexdigest()

        create_branch(repo, branch_name_hash)

        checkout(repo, branch_name_hash)

        repo_path = get_repo_path_from_origin(repo)

        add_drone_cron(repo_path, branch_name_hash, cron)


def append_header_to_drone_yml(header):
    header = "# " + header + "\n"

    drone_yml = open(DRONE_PATH, "r+")

    body = drone_yml.read()

    drone_yml.seek(0)

    drone_yml.write(header + body)

    drone_yml.close()


def commit_changes():
    header = generate_header()

    append_header_to_drone_yml(header)

    repo = get_repo()

    commit(repo, header)

    push(repo)



import os
from shutil import which
from sys import exit

import click

from .__version__ import __version__
from .commands.drone import cli as drone
from .commands.init import cli as init
from .commands.run import cli as run
from .commands.template import cli as template


def check_requirements():
    ok = True

    requirements = [
        "git",
        "drone"
    ]

    for requirement in requirements:
        if which(requirement) is None:
            ok = False
            click.echo(f"Missing tool: {requirement}")

    env_vars = [
        "DRONE_SERVER",
        "DRONE_TOKEN"
    ]

    for var in env_vars:
        if var not in os.environ:
            ok = False
            click.echo(f"Missing environment variable: {var}")

    if not ok:
        click.echo("Exiting...")
        exit()


@click.group()
@click.version_option(prog_name="seki", version=__version__)
def cli():
    check_requirements()


cli.add_command(init)
cli.add_command(drone)
cli.add_command(run)
cli.add_command(template)

if __name__ == "__main__":
    cli()



import click

from .template import get_template_params, get_params_values, process_template
from ...commands.shared import prepare_project, commit_changes
from ...conf import DRONE_PATH


@click.command("template", short_help="Generate drone from template.")
@click.argument("file", type=click.Path(exists=True))
@click.option("--cron", type=click.Choice(["@hourly", "@daily", "@weekly", "@monthly", "@yearly"]))
def cli(file, cron):
    prepare_project(cron)

    params = get_template_params(file)

    replacements = get_params_values(params)

    process_template(file, DRONE_PATH, replacements)

    commit_changes()



import click
import requests

from requests.auth import HTTPBasicAuth

from ...conf import REPOSITORY_PATH
from ...utils.git_cli import clone_repo


@click.command("github")
@click.option("--username", prompt=True)
@click.option("--password", prompt=True, hide_input=True, confirmation_prompt=True)
def command(username, password):
    payload = {
        "name": "seki",
        "private": True,
    }

    auth = HTTPBasicAuth(username, password)

    response = requests.post("https://api.github.com/user/repos", data=payload, auth=auth)

    if response.status_code != requests.codes.created:
        click.echo(response.text)

        click.echo("Try manual activation.")
    else:
        clone_url = f"git@github.com:{username}/seki.git"
        clone_repo(clone_url, REPOSITORY_PATH)



from urllib.parse import urlparse

import click
from git import Repo

from ..conf import REPOSITORY_PATH


def get_repo():
    return Repo(REPOSITORY_PATH)


def clone_repo(clone_url, folder):
    click.echo("Clonning repo...")
    return Repo.clone_from(clone_url, folder)


def create_branch(repo, branch_name):
    click.echo(f"Creating new branch '{branch_name}'...")
    repo.git.reset("--hard")

    repo.git.checkout("-b", branch_name)


def checkout(repo, branch_name):
    repo.git.reset("--hard")

    repo.git.checkout(branch_name)


def commit(repo, message):
    repo.git.add("--all")

    repo.git.commit(m=message)

    short_sha = repo.head.object.hexsha[:7]

    click.echo(f"Commit: '{short_sha}'")


def push(repo):
    click.echo("Pushing changes...")
    branch = repo.active_branch.name
    if branch != "master":
        repo.git.push("origin", branch)
    else:
        repo.git.push()


def get_repo_path_from_origin(repo):
    repo_url = repo.remotes.origin.url
    path = urlparse(repo_url).path
    path = path.rsplit(".", 1)[0]

    if path.startswith("/"):
        path = path[1:]

    return path



import click
from pybitbucket.auth import BasicAuthenticator
from pybitbucket.bitbucket import Client
from pybitbucket.repository import (
    Repository,
    RepositoryPayload,
    RepositoryForkPolicy
)
from requests.exceptions import HTTPError

from ...conf import REPOSITORY_PATH
from ...utils.git_cli import clone_repo


def find_bitbucket_repository(email, user, password):
    bitbucket = Client(
        BasicAuthenticator(user, password, email)
    )

    try:
        click.echo("Finding seki project in bitbucket...")
        repository = Repository.find_repository_by_name_and_owner(
            repository_name="seki",
            client=bitbucket
        )
    except HTTPError:
        click.echo("Project not found")
        click.echo("Creating project seki...")

        repository = Repository.create(
            payload=RepositoryPayload({
                "name": "seki",
                "is_private": True,
                "fork_policy": RepositoryForkPolicy.NO_FORKS,
            }),
            client=bitbucket
        )

    for link in repository.links["clone"]:
        if link["name"] == "https":
            return link["href"]


@click.command("bitbucket")
@click.option("--user", prompt=True)
@click.option("--email", prompt=True)
@click.option("--password", prompt=True, hide_input=True, confirmation_prompt=True)
def command(email, user, password):
    clone_url = find_bitbucket_repository(email, user, password)

    clone_repo(clone_url, REPOSITORY_PATH)



import click
import yaml


def generate_drone_yml(image, args, telegram):
    click.echo("Generating new 'drone.yml' from args...")

    template = {
        "kind": "pipeline",
        "name": "default",
        "clone": {
            "disable": True
        },
        "steps": []
    }

    run_step = {
        "name": "run",
        "image": image,
    }

    if args:
        run_step["commands"] = [args]

    template["steps"].append(run_step)

    if telegram:
        template["steps"].append({
            "name": "create tar",
            "image": "alpine",
            "commands": [
                "tar czf output.tar.gz ."
            ]
        })

        template["steps"].append({
            "name": "telegram notificaton",
            "image": "appleboy/drone-telegram",
            "settings": {
                "token": {
                    "from_secret": "telegram_token"
                },
                "to": {
                    "from_secret": "telegram_to"
                },
                "format": "markdown",
                "message": """
                    {{#success build.status}}
                        {{build.number}}: ✅ `{{commit.message}}` 🚁 [See build]({{build.link}})
                    {{else}}
                        {{build.number}}: ❌ `{{commit.message}}` 🚁 [See build]({{build.link}})
                    {{/success}}
                """,
                "document": [
                    "output.tar.gz"
                ]
            }
        })

    return template


def dump_drone_yml(drone_path, template):
    click.echo("Dumping content to 'drone.yml'")

    drone_yml = open(drone_path, "w")

    yaml.dump(template, drone_yml, default_flow_style=False)

    drone_yml.close()



import re

import click


def get_template_params(template_path):
    template_file = open(template_path, "r")

    first_line = template_file.readline()

    if "# PARAMETERS:" in first_line:
        first_line = first_line.replace("# PARAMETERS:", "")
        return [param.strip() for param in first_line.split(",")]
    else:
        return []


def get_params_values(params):
    replacements = {}

    for param in params:
        key = "$$" + param.upper()
        replacements[key] = input(f"Value for '{param}': ")

    return replacements


def do_replacements_on(text, replacements):
    pattern = re.compile("|".join(re.escape(k) for k in replacements))

    return pattern.sub(lambda match: replacements[match.group(0)], text)


def process_template(template_path, output_path, replacements):
    click.echo(f"Generating 'drone.yml' from '{template_path}'...")

    template_file = open(template_path, "r")
    output_file = open(output_path, "w")

    for line in template_file:
        if replacements:
            line = do_replacements_on(line, replacements)
        output_file.write(line)

    template_file.close()
    output_file.close()
